import os
import sys


def _initializeLibs():
    '''
    Initialize required libraries based on Maya version
    Returns
    '''

    print "Importing DHI external libraries"
    from DHI.modules.file.handler import FileHandler

    current = os.path.dirname(os.path.abspath(__file__))

    if current not in sys.path:
        sys.path.append(current)

    libFolder = FileHandler.joinPath((current, "./lib"))
    print "Fetching libraries from folder ", libFolder

    if libFolder not in sys.path:
        sys.path.append(libFolder)


def _loadRequiredPlugins():
    try:
        from modules.maya.util import MayaUtil
        from modules.file.handler import FileHandler
        from modules.maya.about import AboutEnv
        current = os.path.dirname(os.path.abspath(__file__))

        about = AboutEnv.getInstance()
        print("Running Maya version ", about.version)
        MayaUtil.loadPlugin('embeddedRL4', FileHandler.joinPath((current, "plugins", about.version)))
        MayaUtil.loadPlugin('MayaUE4RBFPlugin2018', FileHandler.joinPath((current, "plugins")))
    except:
        print("Could not load required plugins!")


def load():
    print("Preparing paths and loading DHI Plugins")
    _initializeLibs()
    _loadRequiredPlugins()
