import os
from modules.file.handler import FileHandler
import re
import DHI.core.consts as consts


class CharacterConfig(object):
    def __init__(self, config):
        self.current = os.path.dirname(os.path.abspath(__file__))

        self.dnaPath = self.prepareInput(config["characterAssets"]["dnaPath"])
        self.bodyScenePath = self.prepareInput(config["characterAssets"]["bodyPath"])

        '''
        Each body has its own body mesh name. This information is needed in later steps of character assembly when 
        we are grouping body meshes into their corresponding LOD groups 
        '''
        self.bodyMeshName = self.resolveBodyMeshName(self.prepareInput(config["characterAssets"]["bodyPath"]))
        self.shadersDirPath = config["common"]["shadersDirPath"]
        self.masksDirPath = config["common"]["masksDirPath"]
        self.sceneOrientationStringValue = self.resolveSceneOrientationStringValue(config["common"])
        self.sceneOrientation = self.resolveSceneOrientation()

        '''
        Path to head common maps (specular, diffuse, jitter,...). Assets found here are common for all characters and 
        are part of plugin
        '''
        self.headMapsPath = config["common"]["headMapsPath"]
        self.mapsDirPath = config["characterAssets"]["mapsDirPath"]

        '''
        Head GUI controls
        '''
        self.guiPath = FileHandler.joinPath((self.current, "assets", "head_gui.ma"))

        '''
        Eye controls
        '''
        self.acPath = FileHandler.joinPath((self.current, "assets", "head_ac.ma"))
        self.shaderScenePath = FileHandler.joinPath((self.current, "assets", "head_shader.ma"))
        self.lightScenePath = FileHandler.joinPath((self.current, "assets", "dh_lights.ma"))

    def prepareInput(self, val):
        return val.replace("\\", "/")

    def resolveBodyMeshName(self, bodyPath):
        fileName = bodyPath.split("/")[-1]
        retval = fileName.replace("_rig.ma", "")
        print "resolved body mesh name: ", retval
        return retval

    def resolveSceneOrientationStringValue(self, configCommon):
        orientation = 'z'

        if "sceneOrientation" in configCommon:
            orientation = configCommon["sceneOrientation"]

        return orientation

    def resolveSceneOrientation(self):
        orientation = self.sceneOrientationStringValue

        if orientation == 'x':
            return consts.ORIENT_X
        elif orientation == 'y':
            return consts.ORIENT_Y
        else:
            return consts.ORIENT_Z

    def __str__(self):
        return """
            DNA path: %s
            Body Scene Path: %s
            Body Mesh Name: %s
            Head Shaders Dir Path: %s
            Head Specific Maps Dir Path: %s
            Head Common Maps Dir Path: %s
            Masks dir path: %s
            Eye controls: %s
            Scene Orientation: %s
        """ % (self.dnaPath,
               self.bodyScenePath,
               self.bodyMeshName,
               self.shadersDirPath,
               self.mapsDirPath,
               self.headMapsPath,
               self.masksDirPath,
               self.acPath,
               self.sceneOrientation)
